# Generated by Django 5.0.9 on 2024-12-21 16:29

import pgtrigger.migrations
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("baseapp_chats", "0005_remove_messagestatus_increment_unread_count_and_more"),
    ]

    operations = [
        pgtrigger.migrations.RemoveTrigger(
            model_name="messagestatus",
            name="increment_unread_count",
        ),
        migrations.AddField(
            model_name="chatroom",
            name="is_group",
            field=models.BooleanField(default=False),
        ),
    ]
